using System.Runtime.InteropServices;

/// <summary>
/// In Unity, the render thread is only accesible in C++ using IssuePluginEvent(). This is a helper
/// class to communicate a C# view with its C++ renderer.
/// </summary>
public class NoesisRenderer
{
    /// <summary>
    /// Registers a view in the render thread
    /// </summary>
    public static void RegisterView(Noesis.View view)
    {
        Noesis_RegisterView(view.CPtr, view.GetHashCode());
    }

    /// <summary>
    /// Sends offscreen render commands to native code
    /// </summary>
    public static void RenderOffscreen(Noesis.View view, UnityEngine.Rendering.CommandBuffer commands)
    {
        commands.IssuePluginEvent(_renderOffscreenCallback, view.GetHashCode());
    }

    /// <summary>
    /// Sends offscreen render commands to native code
    /// </summary>
    public static void RenderOffscreen(Noesis.View view)
    {
        UnityEngine.GL.IssuePluginEvent(_renderOffscreenCallback, view.GetHashCode());
    }

    /// <summary>
    /// Sends render commands to native code
    /// </summary>
    public static void RenderOnscreen(Noesis.View view, UnityEngine.Rendering.CommandBuffer commands)
    {
        commands.IssuePluginEvent(_renderOnscreenCallback, view.GetHashCode());
    }

    /// <summary>
    /// Sends render commands to native code
    /// </summary>
    public static void RenderOnscreen(Noesis.View view)
    {
        UnityEngine.GL.IssuePluginEvent(_renderOnscreenCallback, view.GetHashCode());
    }

    /// <summary>
    /// Unregister given renderer
    /// </summary>
    public static void Shutdown(Noesis.View view, UnityEngine.Rendering.CommandBuffer commands)
    {
        commands.IssuePluginEvent(_renderShutdownCallback, view.GetHashCode());
    }

    /// <summary>
    /// Unregister given renderer
    /// </summary>
    public static void Shutdown(Noesis.View view)
    {
        UnityEngine.GL.IssuePluginEvent(_renderShutdownCallback, view.GetHashCode());
    }

    #region Private
#if UNITY_IPHONE || UNITY_XBOX360
    [DllImport("__Internal")]
#else
    [DllImport("Noesis")]
#endif
    private static extern void Noesis_RegisterView(HandleRef renderer, int id);

#if UNITY_IPHONE || UNITY_XBOX360
    [DllImport("__Internal")]
#else
    [DllImport("Noesis")]
#endif
    private static extern System.IntPtr Noesis_GetRenderOffscreenCallback();

#if UNITY_IPHONE || UNITY_XBOX360
    [DllImport("__Internal")]
#else
    [DllImport("Noesis")]
#endif
    private static extern System.IntPtr Noesis_GetRenderOnscreenCallback();

#if UNITY_IPHONE || UNITY_XBOX360
    [DllImport("__Internal")]
#else
    [DllImport("Noesis")]
#endif
    private static extern System.IntPtr Noesis_GetRenderShutdownCallback();

    private static System.IntPtr _renderOffscreenCallback = Noesis_GetRenderOffscreenCallback();
    private static System.IntPtr _renderOnscreenCallback = Noesis_GetRenderOnscreenCallback();
    private static System.IntPtr _renderShutdownCallback = Noesis_GetRenderShutdownCallback();
    #endregion
}