using UnityEngine;
using System.IO;


[PreferBinarySerialization]
public class NoesisXaml: ScriptableObject
{
    void OnDisable()
    {
        UnregisterDependencies();
    }

    public bool CanLoad()
    {
        return string.IsNullOrEmpty(error) && !string.IsNullOrEmpty(source) && content != null && content.Length != 0;
    }

    public object Load()
    {
        if (CanLoad())
        {
            RegisterDependencies();
            return Noesis.GUI.LoadXaml(source);
        }

        throw new System.Exception(string.IsNullOrEmpty(error) ? "Xaml has errors!" : error);
    }

    public void LoadComponent(object component)
    {
        if (CanLoad())
        {
            RegisterDependencies();
            Noesis.GUI.LoadComponent(component, source);
            return;
        }

        throw new System.Exception(string.IsNullOrEmpty(error) ? "Xaml has errors!" : error);
    }

    public void RegisterDependencies()
    {
        if (!_registered && CanLoad())
        {
            NoesisUnity.Init();
            _RegisterDependencies();
            _registered = true;
        }
    }

    public void UnregisterDependencies()
    {
        if (_registered)
        {
            _UnregisterDependencies();
            _registered = false;
        }
    }

    private void _RegisterDependencies()
    {
        NoesisXamlProvider.instance.Register(this);

        if (textures != null && texturePaths != null)
        {
            for (int i = 0; i < texturePaths.Length; i++)
            {
                NoesisTextureProvider.instance.Register(texturePaths[i], textures[i]);
            }
        }

        if (fonts != null)
        {
            foreach (var font in fonts)
            {
                NoesisFontProvider.instance.Register(font);
            }
        }

        if (xamls != null)
        {
            foreach (var xaml in xamls)
            {
                xaml.RegisterDependencies();
            }
        }
    }

    private void _UnregisterDependencies()
    {
        NoesisXamlProvider.instance.Unregister(this);

        if (texturePaths != null)
        {
            foreach (var texture in texturePaths)
            {
                NoesisTextureProvider.instance.Unregister(texture);
            }
        }

        if (fonts != null)
        {
            foreach (var font in fonts)
            {
                NoesisFontProvider.instance.Unregister(font);
            }
        }

        if (xamls != null)
        {
            foreach (var xaml in xamls)
            {
                xaml.UnregisterDependencies();
            }
        }
    }

    public string source;
    public byte[] content;
    public string error;

    public NoesisFont[] fonts;
    public string[] texturePaths;
    public UnityEngine.Texture[] textures;
    public NoesisXaml[] xamls;

    private bool _registered = false;
}
